# import unittest
# import os
# import imp
# from pdt import create_provider_indexes
#
#
# class TestCreateProviderIndexes(unittest.TestCase):
#
#     def test_ensure_provider_indexes(self):
#         """Run ensure_provider_indexes should return results """
#
#         csvfile = os.path.join(os.path.dirname(__file__),  "fiftythousand.csv")
#         outputdir = "outputdir"
#
#         result = create_provider_indexes.ensure_provider_indexes(
#             csvfile, outputdir)
#         print(result)
#
# if __name__ == '__main__':
#     unittest.main()
